@@ -25,12 +25,11 @@
 #include <time.h>
 
 #include <eel/eel.h>
-#include <gpgme.h>
 #include <gnome.h>
 
+#include "seahorse-gpgmex.h"
 #include "seahorse-agent.h"
 #include "seahorse-agent-secmem.h"
-#include "seahorse-util.h"
 
 /*
  * Implementation of the password cache. Note that only passwords
@@ -399,11 +398,14 @@ seahorse_agent_cache_getname (const gchar *id)
     g_return_val_if_fail (g_ctx != NULL, g_strdup (UNKNOWN_KEY));
     g_return_val_if_fail (id != NULL, g_strdup (UNKNOWN_KEY));
         
-    gpgme_get_key (g_ctx, id, &key, 0);
-    if (key == NULL)
-        return g_strdup (UNKNOWN_KEY);
+    gpgme_get_key (g_ctx, id, &key, 1);
+    if (key == NULL) {
+        gpgme_get_key (g_ctx, id, &key, 0);
+        if (key == NULL) 
+            return g_strdup (UNKNOWN_KEY);
+    }
 
-    userid = key->uids && key->uids->uid ? key->uids->uid : UNKNOWN_KEY;
+    userid = (key->uids && key->uids->uid) ? key->uids->uid : UNKNOWN_KEY;
     
     /* If not utf8 valid, assume latin 1 */
     if (!g_utf8_validate (userid, -1, NULL))
