@@ -26,7 +26,7 @@
 
 enum {
 	PROP_0,
-	PROP_CTX,
+	PROP_KEY_SOURCE,
     PROP_MODE,
     PROP_FILTER
 };
@@ -53,17 +53,18 @@ struct _SeahorseKeyStorePriv {
 /* Internal data stored at 0 in the tree store in order to keep track
  * of the location, key-store and key.
  */
-typedef struct
-{
+typedef struct {
 	SeahorseKeyStore	*skstore;
-	GtkTreeRowReference	*ref;
-	SeahorseKey		*skey;
-} SeahorseKeyRow;
+    GPtrArray           *refs;     /* GtkTreeRowReference pointers */
+	SeahorseKey		    *skey;     /* The key we're dealing with */
+} SeahorseKeyRow;                   
 
 static void	seahorse_key_store_class_init		(SeahorseKeyStoreClass	*klass);
 static GObject* seahorse_key_store_constructor  (GType type, guint n_props, 
                                              GObjectConstructParam* props);
-static void	seahorse_key_store_finalize		(GObject		*gobject);
+static void seahorse_key_store_dispose          (GObject       *gobject);
+static void	seahorse_key_store_finalize		    (GObject       *gobject);
+
 static void	seahorse_key_store_set_property		(GObject		*gobject,
 							 guint			prop_id,
 							 const GValue		*value,
@@ -73,42 +74,44 @@ static void	seahorse_key_store_get_property		(GObject		*gobject,
 							 GValue			*value,
 							 GParamSpec		*pspec);
 /* Virtual methods */
-static void	seahorse_key_store_append_key   (SeahorseKeyStore   *skstore,
+static gboolean  seahorse_key_store_append  (SeahorseKeyStore   *skstore,
                                              SeahorseKey        *skey,
+                                             guint              uid,
 	                                         GtkTreeIter        *iter);
-static void	seahorse_key_store_set          (SeahorseKeyStore   *skstore,
+static void	     seahorse_key_store_set     (SeahorseKeyStore   *skstore,
                                              SeahorseKey        *skey,
+                                             guint              uid,
                 							 GtkTreeIter        *iter);
-static void	seahorse_key_store_remove_iter  (SeahorseKeyStore   *skstore,
-                                             GtkTreeIter        *iter);
-static void	seahorse_key_store_changed      (SeahorseKeyStore   *skstore,
+static void	     seahorse_key_store_changed (SeahorseKeyStore   *skstore,
                                              SeahorseKey        *skey,
+                                             guint              uid,
                                              GtkTreeIter        *iter,
                                              SeahorseKeyChange  change);
-/* Context signals */
-static void	seahorse_key_store_context_destroyed	(GtkObject		*object,
-							 SeahorseKeyStore	*skstore);
-static void	seahorse_key_store_key_added		(SeahorseContext	*sctx,
-							 SeahorseKey		*skey,
-							 SeahorseKeyStore	*skstore);
-static void seahorse_key_store_key_removed      (SeahorseContext    *sctx,
-                             SeahorseKey        *skey,
-                             SeahorseKeyStore   *skstore);
+
+/* Key source signals */
+static void seahorse_key_store_key_added    (SeahorseKeySource  *sksrc,
+                                             SeahorseKey        *skey,
+                                             SeahorseKeyStore   *skstore);
+static void seahorse_key_store_key_removed  (SeahorseKeySource  *sksrc,
+                                             SeahorseKey        *skey,
+                                             SeahorseKeyStore   *skstore);
                              
 /* Key signals */
 static void	seahorse_key_store_key_changed		(SeahorseKey		*skey,
 							 SeahorseKeyChange	change,
 							 SeahorseKeyRow		*skrow);
 /* Key Row methods */
-static void	seahorse_key_row_new			(SeahorseKeyStore	*skstore,
-							 GtkTreeIter		*iter,
-							 SeahorseKey		*skey);
-static void	seahorse_key_row_remove			(SeahorseKeyRow		*skrow);
+static void	seahorse_key_row_add			(SeahorseKeyStore	*skstore,
+                                             GtkTreeIter        *iter,
+                                             SeahorseKey        *skey);
+static void seahorse_key_row_remove         (SeahorseKeyRow     *skrow,
+                                             GtkTreeIter        *iter);
+static void seahorse_key_row_remove_all     (SeahorseKeyRow     *skrow);
 
 static void seahorse_key_row_free           (SeahorseKeyRow     *skrow);
 
 /* Filter row method */
-static gboolean filter_callback              (GtkTreeModel *model,
+static gboolean filter_callback             (GtkTreeModel *model,
                                              GtkTreeIter *iter,
                                              gpointer data);
                                              
@@ -146,25 +149,26 @@ seahorse_key_store_class_init (SeahorseKeyStoreClass *klass)
 	gobject_class = G_OBJECT_CLASS (klass);
 	
     gobject_class->constructor = seahorse_key_store_constructor;
-	gobject_class->finalize = seahorse_key_store_finalize;
+    gobject_class->finalize = seahorse_key_store_finalize;
+    gobject_class->dispose = seahorse_key_store_dispose;
 	gobject_class->set_property = seahorse_key_store_set_property;
 	gobject_class->get_property = seahorse_key_store_get_property;
 	
-	klass->append = seahorse_key_store_append_key;
+	klass->append = seahorse_key_store_append;
 	klass->set = seahorse_key_store_set;
-	klass->remove = seahorse_key_store_remove_iter;
 	klass->changed = seahorse_key_store_changed;
   
   	/* Class defaults. Derived classes should override */
     klass->use_check = FALSE;
+    klass->use_icon = FALSE;
     klass->n_columns = KEY_STORE_NCOLS;
     klass->col_types = col_types;
     klass->col_ids = col_ids;
 	
-	g_object_class_install_property (gobject_class, PROP_CTX,
-		g_param_spec_object ("ctx", "Seahorse Context",
-				     "Current Seahorse Context to use",
-				     SEAHORSE_TYPE_CONTEXT, G_PARAM_READWRITE));
+	g_object_class_install_property (gobject_class, PROP_KEY_SOURCE,
+		g_param_spec_object ("key-source", "Seahorse Key Source",
+				     "Current Seahorse Key Source to use",
+				     SEAHORSE_TYPE_KEY_SOURCE, G_PARAM_READWRITE));
                     
     g_object_class_install_property (gobject_class, PROP_MODE,
         g_param_spec_uint ("mode", "Key Store Mode",
@@ -202,6 +206,34 @@ seahorse_key_store_constructor (GType type, guint n_props, GObjectConstructParam
     
     return obj;
 }
+
+/* dispose of all our internal references */
+static void
+seahorse_key_store_dispose (GObject *gobject)
+{
+    SeahorseKeyStore *skstore;
+    
+    /*
+     * Note that after this executes the rest of the object should
+     * still work without a segfault. This basically nullifies the 
+     * object, but doesn't free it.
+     * 
+     * This function should also be able to run multiple times.
+     */
+
+    skstore = SEAHORSE_KEY_STORE (gobject);  
+
+    if (skstore->sksrc) {
+        g_signal_handlers_disconnect_by_func (skstore->sksrc, 
+                                seahorse_key_store_key_added, skstore);
+        g_signal_handlers_disconnect_by_func (skstore->sksrc, 
+                                seahorse_key_store_key_removed, skstore);
+        g_object_unref (skstore->sksrc);        
+        skstore->sksrc = NULL;
+    }
+    
+    G_OBJECT_CLASS (parent_class)->dispose (gobject);
+}
     
 static void
 seahorse_key_store_finalize (GObject *gobject)
@@ -209,13 +241,8 @@ seahorse_key_store_finalize (GObject *gobject)
 	SeahorseKeyStore *skstore;
 	
 	skstore = SEAHORSE_KEY_STORE (gobject);
-	
-	g_signal_handlers_disconnect_by_func (GTK_OBJECT (skstore->sctx),
-		seahorse_key_store_context_destroyed, skstore);
-	g_signal_handlers_disconnect_by_func (skstore->sctx,
-		seahorse_key_store_key_added, skstore);
-	g_object_unref (skstore->sctx);
-
+    g_assert (skstore->sksrc == NULL);
+    	
     /* These were allocated in the constructor */
     g_object_unref (skstore->priv->sort);
     g_object_unref (skstore->priv->filter);
@@ -257,15 +284,14 @@ seahorse_key_store_set_property (GObject *gobject, guint prop_id,
 	skstore = SEAHORSE_KEY_STORE (gobject);
 	
 	switch (prop_id) {
-		/* Connects to context signals */
-		case PROP_CTX:
-			skstore->sctx = g_value_get_object (value);
-			g_object_ref (skstore->sctx);
-			g_signal_connect_after (skstore->sctx, "destroy",
-				G_CALLBACK (seahorse_key_store_context_destroyed), skstore);
-            g_signal_connect_after (skstore->sctx, "added",
+		/* Connects to key source signals */
+		case PROP_KEY_SOURCE:
+            g_return_if_fail (skstore->sksrc == NULL);
+			skstore->sksrc = g_value_get_object (value);
+			g_object_ref (skstore->sksrc);
+            g_signal_connect_after (skstore->sksrc, "added",
                 G_CALLBACK (seahorse_key_store_key_added), skstore);
-            g_signal_connect_after (skstore->sctx, "removed",
+            g_signal_connect_after (skstore->sksrc, "removed",
                 G_CALLBACK (seahorse_key_store_key_removed), skstore);
 			break;
 			
@@ -309,8 +335,8 @@ seahorse_key_store_get_property (GObject *gobject, guint prop_id,
 	skstore = SEAHORSE_KEY_STORE (gobject);
 	
 	switch (prop_id) {
-		case PROP_CTX:
-			g_value_set_object (value, skstore->sctx);
+		case PROP_KEY_SOURCE:
+			g_value_set_object (value, skstore->sksrc);
 			break;
 			
 		/* The filtering mode */
@@ -331,86 +357,109 @@ seahorse_key_store_get_property (GObject *gobject, guint prop_id,
 }
 
 /* Sets attributes then appends a new #SeahorseKeyRow */
-static void
-seahorse_key_store_append_key (SeahorseKeyStore *skstore, SeahorseKey *skey, 
-                                    GtkTreeIter *iter)
+static gboolean
+seahorse_key_store_append (SeahorseKeyStore *skstore, SeahorseKey *skey, 
+                           guint uid, GtkTreeIter *iter)
 {
-	SEAHORSE_KEY_STORE_GET_CLASS (skstore)->set (skstore, skey, iter);
-	seahorse_key_row_new (skstore, iter, skey);
+	SEAHORSE_KEY_STORE_GET_CLASS (skstore)->set (skstore, skey, uid, iter);
+	seahorse_key_row_add (skstore, iter, skey);
+    return FALSE;
 }
 
 /* Sets Name and KeyID */
 static void
 seahorse_key_store_set (SeahorseKeyStore *skstore, SeahorseKey *skey, 
-                                    GtkTreeIter *iter)
+                        guint uid, GtkTreeIter *iter)
 {
-    gchar *userid = seahorse_key_get_userid (skey, 0);
+    gchar *userid = seahorse_key_get_userid (skey, uid);
+    gboolean pair = SEAHORSE_IS_KEY_PAIR(skey);
 	gtk_tree_store_set (GTK_TREE_STORE (skstore), iter,
         KEY_STORE_CHECK, FALSE,
+        KEY_STORE_PAIR, uid == 0 ? pair : FALSE,
+        KEY_STORE_NOTPAIR, uid == 0 ? !pair : FALSE,
 		KEY_STORE_NAME, userid,
-		KEY_STORE_KEYID, seahorse_key_get_keyid (skey, 0), -1);
+		KEY_STORE_KEYID, seahorse_key_get_keyid (skey, 0),
+        KEY_STORE_UID, uid, -1);
     g_free (userid);
 }
 
-/* Removes row at @iter from store */
-static void
-seahorse_key_store_remove_iter (SeahorseKeyStore *skstore, GtkTreeIter *iter)
-{
-	gtk_tree_store_remove (GTK_TREE_STORE(skstore), iter);
-}
-
 /* Refreshes key if uids have changed */
 static void
-seahorse_key_store_changed (SeahorseKeyStore *skstore, SeahorseKey *skey, 
-                                GtkTreeIter *iter, SeahorseKeyChange change)
+seahorse_key_store_changed (SeahorseKeyStore *skstore, SeahorseKey *skey, guint uid,
+                            GtkTreeIter *iter, SeahorseKeyChange change)
 {
 	switch (change) {
-        case SKEY_CHANGE_ALL:
+        case SKEY_CHANGE_ALL:     
 		case SKEY_CHANGE_UIDS:
-			SEAHORSE_KEY_STORE_GET_CLASS (skstore)->set (skstore, skey, iter);
+			SEAHORSE_KEY_STORE_GET_CLASS (skstore)->set (skstore, skey, uid, iter);
 			break;
 		default:
 			break;
 	}
 }
 
-/* Destroys @skstore */
+/* Appends @skey and all uids */
 static void
-seahorse_key_store_context_destroyed (GtkObject *object, SeahorseKeyStore *skstore)
-{
-	seahorse_key_store_destroy (skstore);
-}
-
-/* Appends @skey */
-static void
-seahorse_key_store_key_added (SeahorseContext *sctx, SeahorseKey *skey, SeahorseKeyStore *skstore)
+seahorse_key_store_key_added (SeahorseKeySource *sksrc, SeahorseKey *skey, SeahorseKeyStore *skstore)
 {
 	GtkTreeIter iter;
-	
-	SEAHORSE_KEY_STORE_GET_CLASS (skstore)->append (skstore, skey, &iter);
+	guint i, uids = seahorse_key_get_num_uids (skey);
+ 
+    for (i = 0; i < uids; i++) {
+    	if(!SEAHORSE_KEY_STORE_GET_CLASS (skstore)->append (skstore, skey, i, &iter))
+            break;
+    }
 }
 
 /* Removes @skrow */
 static void
-seahorse_key_store_key_removed (SeahorseContext *sctx, SeahorseKey *skey, SeahorseKeyStore *skstore)
+seahorse_key_store_key_removed (SeahorseKeySource *sksrc, SeahorseKey *skey, SeahorseKeyStore *skstore)
 {
-    SeahorseKeyRow *skrow;
-    
-    skrow = (SeahorseKeyRow*)g_hash_table_lookup (skstore->priv->rows, skey);
-	seahorse_key_row_remove (skrow);
+    SeahorseKeyRow *skrow = (SeahorseKeyRow*)g_hash_table_lookup (skstore->priv->rows, skey);
+	seahorse_key_row_remove_all (skrow);
 }
 
-/* Gets location of @skey, then calls virtual changed() */
+/* Calls virtual |changed| for all relevant uids. adds new uids if necessary */
 static void
 seahorse_key_store_key_changed (SeahorseKey *skey, SeahorseKeyChange change, SeahorseKeyRow *skrow)
 {
-	GtkTreePath *path;
-	GtkTreeIter iter;
+    guint i, uid, old_uids, num_uids;
+    GtkTreeIter first;
+    GtkTreeIter iter;
+    GtkTreePath *path;
+    
+    old_uids = 0;
+    num_uids = seahorse_key_get_num_uids (skey);
+    
+    for (i = 0; i < skrow->refs->len; i++) {
+        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
+        path = gtk_tree_row_reference_get_path ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
+        g_return_if_fail (gtk_tree_model_get_iter (GTK_TREE_MODEL (skrow->skstore), &iter, path));
+        gtk_tree_path_free (path);        
 
-	path = gtk_tree_row_reference_get_path (skrow->ref);
-	g_return_if_fail (gtk_tree_model_get_iter (GTK_TREE_MODEL (skrow->skstore), &iter, path));
-	SEAHORSE_KEY_STORE_GET_CLASS (skrow->skstore)->changed (skrow->skstore, skey, &iter, change);
-    gtk_tree_path_free (path);
+        gtk_tree_model_get (GTK_TREE_MODEL (skrow->skstore), &iter, KEY_STORE_UID, &uid, -1);
+        
+        /* Remove any extra rows */
+        if (uid >= num_uids) {
+            seahorse_key_row_remove (skrow, &iter);
+            i--;
+            continue;
+        }
+
+        SEAHORSE_KEY_STORE_GET_CLASS (skrow->skstore)->changed (skrow->skstore, skey, uid, &iter, change);
+
+        /* The top parent row */
+        if (uid == 0)
+            memcpy (&first, &iter, sizeof (first));            
+
+        /* Find the max uid on the rows */
+        if (uid >= old_uids)
+            old_uids = uid + 1;
+    }
+
+    /* Add all new rows */    
+    for (i = old_uids; i < num_uids; i++)
+        SEAHORSE_KEY_STORE_GET_CLASS (skrow->skstore)->append (skrow->skstore, skey, i, &first);
 }
 
 /* Update the sort order for a column */
@@ -484,12 +533,31 @@ sort_changed(GtkTreeSortable *sort, gpointer user_data)
     }
 }
 
+/* Try to find our key store given a tree model */
+static SeahorseKeyStore* 
+key_store_from_model (GtkTreeModel *model)
+{
+	/* Sort models are what's set on the tree */
+    if (GTK_IS_TREE_MODEL_SORT (model)) {
+        model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
+        model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
+    }
+    
+    if (SEAHORSE_IS_KEY_STORE (model))
+        return SEAHORSE_KEY_STORE (model);
+    
+    g_assert_not_reached ();
+    return NULL;
+}
+
 /* Called when a checkbox is toggled, toggle the value */
 static void
 check_toggled(GtkCellRendererToggle *cellrenderertoggle, gchar *path, gpointer user_data)
 {
-    SeahorseKeyStore *skstore = SEAHORSE_KEY_STORE (user_data);
+    GtkTreeView *view = GTK_TREE_VIEW (user_data);
+    SeahorseKeyStore *skstore = key_store_from_model (gtk_tree_view_get_model (view));
     GtkTreeModel* fmodel = GTK_TREE_MODEL (skstore->priv->sort);
+    GtkTreeSelection *selection;
     gboolean prev = FALSE;
     GtkTreeIter iter;
     GtkTreeIter child;
@@ -508,43 +576,59 @@ check_toggled(GtkCellRendererToggle *cellrenderertoggle, gchar *path, gpointer u
     g_value_unset (&v);    
     
     gtk_tree_store_set (GTK_TREE_STORE (skstore), &child, KEY_STORE_CHECK, prev ? FALSE : TRUE, -1);
+
+    selection = gtk_tree_view_get_selection (view);
+    g_signal_emit_by_name (selection, "changed");    
 }
 
 /* Creates a new #SeahorseKeyRow for listening to key signals */
 static void
-seahorse_key_row_new (SeahorseKeyStore *skstore, GtkTreeIter *iter, SeahorseKey *skey)
+seahorse_key_row_add (SeahorseKeyStore *skstore, GtkTreeIter *iter, SeahorseKey *skey)
 {
 	SeahorseKeyRow *skrow;
 	GtkTreePath *path;
+    
+    /* Do we already have a row for this key? */
+    skrow = (SeahorseKeyRow*)g_hash_table_lookup (skstore->priv->rows, skey);
+    
+    if (!skrow) {
+        skrow = g_new0 (SeahorseKeyRow, 1);
+        skrow->refs = g_ptr_array_new ();
+        skrow->skstore = skstore;
 
-	skrow = g_new0 (SeahorseKeyRow, 1);
-	skrow->skstore = skstore;
+        skrow->skey = skey;
+        g_object_ref (skey);
 
-	path = gtk_tree_model_get_path (GTK_TREE_MODEL (skstore), iter);
-	skrow->ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (skstore), path);
+        g_signal_connect_after (skrow->skey, "changed",
+                                G_CALLBACK (seahorse_key_store_key_changed), skrow);
+
+        /* Put it in our row cache */
+        g_hash_table_replace (skstore->priv->rows, skey, skrow);
+    }
+    
+    path = gtk_tree_model_get_path (GTK_TREE_MODEL (skstore), iter);
+    g_ptr_array_add (skrow->refs, gtk_tree_row_reference_new (GTK_TREE_MODEL (skstore), path));
     gtk_tree_path_free (path);
     
-	skrow->skey = skey;
-	g_object_ref (skey);
-	g_signal_connect_after (skrow->skey, "changed",
-		G_CALLBACK (seahorse_key_store_key_changed), skrow);
-
-    /* Put it in our row cache */
-    g_hash_table_replace (skstore->priv->rows, skey, skrow);
-
     gtk_tree_store_set (GTK_TREE_STORE (skstore), iter, KEY_STORE_DATA, skrow, -1);
 }
 
 static void
 seahorse_key_row_free (SeahorseKeyRow *skrow)
 {
+    guint i;
+    
     /* Unref key */
     g_signal_handlers_disconnect_by_func (skrow->skey,
-             seahorse_key_store_key_changed, skrow);
+                                          seahorse_key_store_key_changed, skrow);
     g_object_unref (skrow->skey);
   
-    gtk_tree_row_reference_free (skrow->ref);
-   
+    for (i = 0; i < skrow->refs->len; i++) {
+        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
+        gtk_tree_row_reference_free ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
+    }
+    g_ptr_array_free (skrow->refs, TRUE);
+    
     g_free (skrow);
 }    
 
@@ -552,26 +636,58 @@ seahorse_key_row_free (SeahorseKeyRow *skrow)
  * key, then frees itself.
  */
 static void
-seahorse_key_row_remove (SeahorseKeyRow *skrow)
+seahorse_key_row_remove_all (SeahorseKeyRow *skrow)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
-
-	path = gtk_tree_row_reference_get_path (skrow->ref);
-	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (skrow->skstore), &iter, path))
-		SEAHORSE_KEY_STORE_GET_CLASS (skrow->skstore)->remove (skrow->skstore, &iter);
-	gtk_tree_path_free (path);
+    guint i;
+    
+    for (i = 0; i < skrow->refs->len; i++) {
+        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
+        path = gtk_tree_row_reference_get_path ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
+        
+        /* Note that removing a parent row could remove it's sub rows, so ... */
+        if (path) {
+            if(gtk_tree_model_get_iter (GTK_TREE_MODEL (skrow->skstore), &iter, path))
+                gtk_tree_store_remove (GTK_TREE_STORE (skrow->skstore), &iter);
+            gtk_tree_path_free (path);
+        }
+    }
 
     /* This also frees the skrow */
-    g_return_if_fail(g_hash_table_remove (skrow->skstore->priv->rows, skrow->skey));
+    g_return_if_fail (g_hash_table_remove (skrow->skstore->priv->rows, skrow->skey));
+}
+
+static void
+seahorse_key_row_remove (SeahorseKeyRow *skrow, GtkTreeIter *iter)
+{
+    GtkTreePath *p1, *p2;
+    guint i, r;
+    
+    p1 = gtk_tree_model_get_path (GTK_TREE_MODEL (skrow->skstore), iter);
+    
+    for (i = 0; i < skrow->refs->len; i++) {
+        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
+        p2 = gtk_tree_row_reference_get_path ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
+        r = gtk_tree_path_compare (p1, p2);
+        gtk_tree_path_free (p2);
+        
+        if (r == 0) {
+            g_ptr_array_remove_index (skrow->refs, i);
+            gtk_tree_store_remove (GTK_TREE_STORE (skrow->skstore), iter);
+            break;
+        }
+    }
+    
+    /* This also frees the skrow */
+    if (skrow->refs->len == 0)
+        g_return_if_fail (g_hash_table_remove (skrow->skstore->priv->rows, skrow->skey));
 }
 
 /**
  * seahorse_key_store_init:
  * @skstore: #SeahorseKeyStore to initialize
  * @view: #GtkTreeView that will show @skstore
- * @cols: Number of columns to be in @view
- * @columns: Array of column types for @skstore
  *
  * Initializes @skstore with default columns and embeds in @view.
  * This must be called after creating a new #SeahorseKeyStore.
@@ -591,11 +707,45 @@ seahorse_key_store_init (SeahorseKeyStore *skstore, GtkTreeView *view)
     if (SEAHORSE_KEY_STORE_GET_CLASS (skstore)->use_check) {
         GtkCellRenderer *renderer;
         renderer = gtk_cell_renderer_toggle_new ();
-        g_signal_connect (renderer, "toggled", G_CALLBACK (check_toggled), skstore);
+        g_signal_connect (renderer, "toggled", G_CALLBACK (check_toggled), view);
         col = gtk_tree_view_column_new_with_attributes ("", renderer, "active", KEY_STORE_CHECK, NULL);
         gtk_tree_view_column_set_resizable (col, FALSE);
         gtk_tree_view_append_column (view, col);
     }
+    
+    /* When using key pair icons, we add an icon column */
+    if (SEAHORSE_KEY_STORE_GET_CLASS (skstore)->use_icon) {
+    	GtkCellRenderer *secretPixbufRenderer = NULL, *keyPixbufRenderer = NULL;
+    	GdkPixbuf *secret_pixbuf = NULL, *key_pixbuf;
+    	
+    	secret_pixbuf = gdk_pixbuf_new_from_file(PIXMAPSDIR "seahorse-secret.png", NULL);
+    	secretPixbufRenderer = gtk_cell_renderer_pixbuf_new();
+    	g_object_set(secretPixbufRenderer,
+    				 "pixbuf", GDK_PIXBUF(secret_pixbuf),
+    				 NULL);
+    	
+    	key_pixbuf = gdk_pixbuf_new_from_file(PIXMAPSDIR "seahorse-key.png", NULL);
+    	keyPixbufRenderer = gtk_cell_renderer_pixbuf_new();
+    	g_object_set(keyPixbufRenderer,
+    				 "pixbuf", GDK_PIXBUF(key_pixbuf),
+    				 NULL);
+    				 			 
+    	col = gtk_tree_view_column_new_with_attributes ("", secretPixbufRenderer, 
+   												"visible", KEY_STORE_PAIR,
+   												NULL);
+
+   		gtk_tree_view_column_pack_end(col,
+   									  keyPixbufRenderer,
+   									  FALSE);
+   									  
+   		gtk_tree_view_column_add_attribute(col, keyPixbufRenderer,
+   												"visible",
+   												KEY_STORE_NOTPAIR); 										
+        gtk_tree_view_column_set_resizable (col, FALSE);
+        gtk_tree_view_append_column (view, col);
+        
+        gtk_tree_view_column_set_sort_column_id (col, KEY_STORE_PAIR);
+      }
       
 	col = seahorse_key_store_append_column (view, _("Name"), KEY_STORE_NAME);
 	gtk_tree_view_column_set_sort_column_id (col, KEY_STORE_NAME);
@@ -608,30 +758,6 @@ seahorse_key_store_init (SeahorseKeyStore *skstore, GtkTreeView *view)
     }
 }
 
-/**
- * seahorse_key_store_destroy:
- * @skstore: #SeahorseKeyStore to destroy
- *
- * Empties @skstore in order to unref any keys it contains.
- * Also unrefs @skstore.  Call this method if @skstore does not exist
- * for the life of the program.
- **/
-void
-seahorse_key_store_destroy (SeahorseKeyStore *skstore)
-{
-	SeahorseKeyRow *skrow;
-	GtkTreeIter iter;
-	
-	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (skstore), &iter)) {
-		do {
-			gtk_tree_model_get (GTK_TREE_MODEL (skstore), &iter, KEY_STORE_DATA, &skrow, -1);
-			seahorse_key_row_remove (skrow);
-		} while (GTK_IS_TREE_MODEL (skstore) &&
-			 gtk_tree_model_get_iter_first (GTK_TREE_MODEL (skstore), &iter));
-	}
-	g_object_unref (skstore);
-}
-
 /**
  * seahorse_key_store_populate:
  * @skstore: #SeahorseKeyStore to populate with #SeahorseKeys
@@ -646,51 +772,31 @@ seahorse_key_store_populate (SeahorseKeyStore *skstore)
 	SeahorseKey *skey;
 	guint count = 1;
 	gdouble length;
-	GtkTreeIter iter;
 	
-	g_return_if_fail (SEAHORSE_IS_KEY_STORE (skstore));
+    g_return_if_fail (SEAHORSE_IS_KEY_STORE (skstore));
+    g_return_if_fail (SEAHORSE_IS_KEY_SOURCE (skstore->sksrc));
 	
     /* Don't precipitate a load */
-    if (seahorse_context_get_n_keys (skstore->sctx) > 0) {
+    if (seahorse_key_source_get_count (skstore->sksrc, FALSE) > 0) {
 
-    	keys = list = seahorse_context_get_keys (skstore->sctx);
+    	keys = list = seahorse_key_source_get_keys (skstore->sksrc, FALSE);
     	length = g_list_length (list);
 	
     	while (list != NULL && (skey = list->data) != NULL) {
-	       	SEAHORSE_KEY_STORE_GET_CLASS (skstore)->append (skstore, skey, &iter);
+            seahorse_key_store_key_added (skstore->sksrc, skey, skstore);
     		list = g_list_next (list);
     		count++;
     	}
      
         g_list_free (keys);
-	
-    	seahorse_context_show_progress (skstore->sctx,
-	       	g_strdup_printf (_("Listed %d keys"), count), -1);
     }
 }
 
-/* Try to find our key store given a tree model */
-static SeahorseKeyStore* 
-key_store_from_model (GtkTreeModel *model)
-{
-	/* Sort models are what's set on the tree */
-    if (GTK_IS_TREE_MODEL_SORT (model)) {
-        model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
-        model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
-    }
-    
-    if (SEAHORSE_IS_KEY_STORE (model))
-        return SEAHORSE_KEY_STORE (model);
-    
-    g_assert_not_reached ();
-    return NULL;
-}
-
 /* Given an iterator find the associated key */
 static SeahorseKey*
-key_from_iterator (GtkTreeModel* model, GtkTreeIter* iter)
+key_from_iterator (GtkTreeModel* model, GtkTreeIter* iter, guint *uid)
 {
-    GtkTreeIter parent, i;
+    GtkTreeIter i;
     SeahorseKeyRow *skrow;
     
     /* Convert to base iter if necessary */
@@ -702,10 +808,9 @@ key_from_iterator (GtkTreeModel* model, GtkTreeIter* iter)
         model = GTK_TREE_MODEL (skstore);
     }
     
-    gtk_tree_model_get (model, 
-        gtk_tree_model_iter_parent (model, &parent, iter) ? &parent : iter, 
-        KEY_STORE_DATA, &skrow, -1);    
-    
+    gtk_tree_model_get (model, iter, 
+        KEY_STORE_DATA, &skrow, uid ? KEY_STORE_UID : -1, uid, -1);    
+
    return skrow->skey;
 }
 
@@ -721,7 +826,7 @@ key_from_iterator (GtkTreeModel* model, GtkTreeIter* iter)
  * Returns: The #SeahorseKey at @path in @view
  **/
 SeahorseKey*
-seahorse_key_store_get_key_from_path (GtkTreeView *view, GtkTreePath *path)
+seahorse_key_store_get_key_from_path (GtkTreeView *view, GtkTreePath *path, guint *uid)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -731,7 +836,7 @@ seahorse_key_store_get_key_from_path (GtkTreeView *view, GtkTreePath *path)
 	
 	model = gtk_tree_view_get_model (view);
 	g_return_val_if_fail (gtk_tree_model_get_iter (model, &iter, path), NULL);
-    return key_from_iterator (model, &iter);
+    return key_from_iterator (model, &iter, uid);
 }
 
 /**
@@ -773,6 +878,25 @@ seahorse_key_store_append_column (GtkTreeView *view, const gchar *label, const g
 	return column;
 }
 
+static gint 
+compare_pointers (gconstpointer a, gconstpointer b)
+{
+    if (a == b)
+        return 0;
+    return a > b ? 1 : -1;
+}
+
+GList*              
+seahorse_key_store_get_all_keys (GtkTreeView *view)
+{
+    SeahorseKeyStore* skstore;
+    
+    g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
+    skstore = key_store_from_model (gtk_tree_view_get_model (view));
+    
+    return seahorse_key_source_get_keys (skstore->sksrc, FALSE);
+}
+
 /**
  * seahorse_key_store_get_selected_keys:
  * @view: #GtkTreeView with selection
@@ -784,7 +908,7 @@ seahorse_key_store_append_column (GtkTreeView *view, const gchar *label, const g
 GList*
 seahorse_key_store_get_selected_keys (GtkTreeView *view)
 {
-	GList *keys = NULL;
+	GList *l, *keys = NULL;
     SeahorseKeyStore* skstore;
     
     g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
@@ -800,7 +924,7 @@ seahorse_key_store_get_selected_keys (GtkTreeView *view)
                 check = FALSE;
                 gtk_tree_model_get (model, &iter, KEY_STORE_CHECK, &check, -1); 
                 if (check)
-                    keys = g_list_append (keys, key_from_iterator (model, &iter));
+                    keys = g_list_append (keys, key_from_iterator (model, &iter, NULL));
             } while (gtk_tree_model_iter_next (model, &iter));
         }
     }
@@ -812,16 +936,22 @@ seahorse_key_store_get_selected_keys (GtkTreeView *view)
         
     	selection = gtk_tree_view_get_selection (view);
     	paths = gtk_tree_selection_get_selected_rows (selection, NULL);
-	    g_return_val_if_fail (paths != NULL && g_list_length (paths) > 0, NULL);
 	
     	/* make key list */
 	    for (list = paths; list != NULL; list = g_list_next (list))
-		    keys = g_list_append (keys, seahorse_key_store_get_key_from_path (view, list->data));
+		    keys = g_list_append (keys, seahorse_key_store_get_key_from_path (view, list->data, NULL));
             
-	   /* free selected paths */
-	   g_list_foreach (paths, (GFunc)gtk_tree_path_free, NULL);
-	   g_list_free (paths);
+        /* free selected paths */
+        g_list_foreach (paths, (GFunc)gtk_tree_path_free, NULL);
+        g_list_free (paths);
     }
+    
+    /* Remove duplicates */
+    keys = g_list_sort (keys, compare_pointers);
+    for (l = keys; l; l = g_list_next (l)) {
+        while (l->next && l->data == l->next->data)
+            keys = g_list_delete_link (keys, l->next);
+    }    
 	 
     return keys;
 }
@@ -829,6 +959,7 @@ seahorse_key_store_get_selected_keys (GtkTreeView *view)
 /**
  * seahorse_key_store_get_selected_key:
  * @view: #GtkTreeView with selection
+ * @uid: Optionally return the selected uid here
  *
  * Sugar method for getting the selected #SeahorseKey from @view.
  *
@@ -836,22 +967,48 @@ seahorse_key_store_get_selected_keys (GtkTreeView *view)
  * more than one selection.
  **/
 SeahorseKey*
-seahorse_key_store_get_selected_key (GtkTreeView *view)
+seahorse_key_store_get_selected_key (GtkTreeView *view, guint *uid)
 {
-	GList *list = NULL;
-	SeahorseKey *skey;
+    SeahorseKeyStore* skstore;
+	SeahorseKey *skey = NULL;
 	
-	g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
-	
-	/* get selected keys */
-	list = seahorse_key_store_get_selected_keys (view);
-	g_return_val_if_fail (list != NULL, NULL);
-	g_return_val_if_fail (g_list_length (list) == 1, NULL);
-	
-	/* get first key, free list */
-	skey = list->data;
-	g_list_free (list);
+    g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
+    skstore = key_store_from_model (gtk_tree_view_get_model (view));
+    
+    if (SEAHORSE_KEY_STORE_GET_CLASS (skstore)->use_check) {
+        GtkTreeModel* model = GTK_TREE_MODEL (skstore);
+        GtkTreeIter iter;
+        gboolean check;
+            
+        if (gtk_tree_model_get_iter_first (model, &iter)) {
+            do {
+                check = FALSE;
+                gtk_tree_model_get (model, &iter, KEY_STORE_CHECK, &check, -1); 
+                if (check) {
+					skey = key_from_iterator (model, &iter, uid);
+					break;
+				}
+            } while (gtk_tree_model_iter_next (model, &iter));
+        }
+    }
+
+    /* Fall back if none checked, or not using checks */
+    if (skey == NULL) {
+    	GList *paths = NULL;
+        GtkTreeSelection *selection;	
+        
+    	selection = gtk_tree_view_get_selection (view);
+    	paths = gtk_tree_selection_get_selected_rows (selection, NULL);
 	
+    	/* make key list */
+		if (paths != NULL)
+		    skey = seahorse_key_store_get_key_from_path (view, paths->data, uid);
+            
+        /* free selected paths */
+        g_list_foreach (paths, (GFunc)gtk_tree_path_free, NULL);
+        g_list_free (paths);
+    }
+		
 	return skey;
 }
 
