@@ -92,7 +92,37 @@ get_ldap_server_info (SeahorseLDAPSource *lsrc, gboolean force)
  
 #define LDAP_ERROR_DOMAIN (get_ldap_error_domain())
 
-GQuark
+#ifdef _DEBUG
+
+static void
+dump_ldap_entry (LDAP *ld, LDAPMessage *res)
+{
+    BerElement *pos;
+    char **values;
+    char **v;
+    char *t;
+    
+    t = ldap_get_dn (ld, res);
+    g_printerr ("dn: %s\n", t);
+    ldap_memfree (t);
+    
+    for (t = ldap_first_attribute (ld, res, &pos); t; 
+         t = ldap_next_attribute (ld, res, pos)) {
+             
+        values = ldap_get_values (ld, res, t);
+        for (v = values; *v; v++) 
+            g_printerr ("%s: %s\n", t, *v);
+             
+        ldap_value_free (values);
+        ldap_memfree (t);
+    }
+    
+    ber_free (pos, 0);
+}
+
+#endif
+
+static GQuark
 get_ldap_error_domain ()
 {
     static GQuark q = 0;
@@ -259,6 +289,7 @@ IMPLEMENT_OPERATION (LDAP, ldap)
  
 static const char *kServerAttributes[] = {
     "basekeyspacedn",
+    "pgpbasekeyspacedn",
     "version",
     NULL
 };
@@ -336,11 +367,16 @@ seahorse_ldap_operation_cancel (SeahorseOperation *operation)
 static void
 fail_ldap_operation (SeahorseLDAPOperation *lop, int code)
 {
+    gchar *t;
+    
     if (code == 0)
         ldap_get_option (lop->ldap, LDAP_OPT_ERROR_NUMBER, &code);
-        
+    
+    g_object_get (lop->lsrc, "key-server", &t, NULL);
     seahorse_operation_mark_done (SEAHORSE_OPERATION (lop), FALSE, 
-            g_error_new_literal (LDAP_ERROR_DOMAIN, code, ldap_err2string(code)));
+            g_error_new (LDAP_ERROR_DOMAIN, code, _("Couldn't communicate with '%s': %s"), 
+                         t, ldap_err2string(code)));
+    g_free (t);
 }
 
 /* Gets called regularly to check for results of LDAP async work */
@@ -410,9 +446,16 @@ done_info_start_op (SeahorseOperation *op, LDAPMessage *result)
         
         /* If we have results then fill in the server info */
         if (r == LDAP_RES_SEARCH_ENTRY) {
+#ifdef _DEBUG
+            g_printerr ("[ldap] Server Info Result:\n");
+            dump_ldap_entry (lop->ldap, result);
+#endif      
+            /* NOTE: When adding attributes here make sure to add them to kServerAttributes */
             sinfo = g_new0 (LDAPServerInfo, 1);
             sinfo->version = get_int_attribute (lop->ldap, result, "version");
             sinfo->base_dn = get_string_attribute (lop->ldap, result, "basekeyspacedn");
+            if (!sinfo->base_dn)
+                sinfo->base_dn = get_string_attribute (lop->ldap, result, "pgpbasekeyspacedn");
             sinfo->key_attr = g_strdup (sinfo->version > 1 ? "pgpkeyv2" : "pgpkey");
             set_ldap_server_info (lop->lsrc, sinfo);
             
@@ -484,10 +527,12 @@ done_bind_start_info (SeahorseOperation *op, LDAPMessage *result)
 
 /* Start an LDAP (bind, server info) request */
 static SeahorseLDAPOperation*
-seahorse_ldap_operation_start (SeahorseLDAPSource *lsrc, OpLDAPCallback cb)
+seahorse_ldap_operation_start (SeahorseLDAPSource *lsrc, OpLDAPCallback cb,
+                               guint total)
 {
     SeahorseLDAPOperation *lop;
     gchar *server = NULL;
+    gchar *t;
 
     g_return_val_if_fail (SEAHORSE_IS_LDAP_SOURCE (lsrc), NULL);
     
@@ -501,13 +546,17 @@ seahorse_ldap_operation_start (SeahorseLDAPSource *lsrc, OpLDAPCallback cb)
     lop->ldap = ldap_init (server, LDAP_PORT);
     g_return_val_if_fail (lop->ldap != NULL, NULL);
 
-    g_free (server);
-
     lop->ldap_cb = done_bind_start_info;
     lop->chain_cb = cb;
 
     seahorse_operation_mark_start (SEAHORSE_OPERATION (lop));
     
+    t = g_strdup_printf (_("Connecting to: %s"), server);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0, total);
+    g_free (t);
+
+    g_free (server);
+    
     /* Start the bind operation */
     lop->ldap_op = ldap_simple_bind (lop->ldap, NULL, NULL);
     if (lop->ldap_op == -1) 
@@ -589,6 +638,10 @@ search_entry (SeahorseOperation *op, LDAPMessage *result)
      
     /* An LDAP entry */
     if (r == LDAP_RES_SEARCH_ENTRY) {
+#ifdef _DEBUG
+            g_printerr ("[ldap] Retrieved Key Entry:\n");
+            dump_ldap_entry (lop->ldap, result);
+#endif      
         parse_key_from_ldap_entry (lop, result);
         return TRUE;
         
@@ -597,10 +650,28 @@ search_entry (SeahorseOperation *op, LDAPMessage *result)
         lop->ldap_op = -1;
         r = ldap_parse_result (lop->ldap, result, &code, NULL, &message, NULL, NULL, 0);
         g_return_val_if_fail (r == LDAP_SUCCESS, FALSE);
-
-        seahorse_operation_mark_done (SEAHORSE_OPERATION (lop), FALSE, 
-                        code == LDAP_SUCCESS ? NULL : g_error_new_literal (LDAP_ERROR_DOMAIN, code, message));
+        
+        /* Error codes that we ignore */
+        switch (code) {
+        case LDAP_SIZELIMIT_EXCEEDED:
+            code = LDAP_SUCCESS;
+            break;
+        };
+        
+        /* Failure */
+        if (code != LDAP_SUCCESS) {
+            if (!message || !message[0]) 
+                fail_ldap_operation (lop, code);
+            else
+                seahorse_operation_mark_done (SEAHORSE_OPERATION (lop), FALSE, 
+                            g_error_new_literal (LDAP_ERROR_DOMAIN, code, message));
+            
+        /* Success */
+        } else {
+            seahorse_operation_mark_done (SEAHORSE_OPERATION (lop), FALSE, NULL);
+        }
         ldap_memfree (message);
+
         return FALSE;
     }       
 }
@@ -611,13 +682,16 @@ start_search (SeahorseOperation *op, LDAPMessage *result)
 {
     SeahorseLDAPOperation *lop = SEAHORSE_LDAP_OPERATION (op);  
     LDAPServerInfo *sinfo;
-    gchar *filter;
+    gchar *filter, *t;
     
     g_return_val_if_fail (lop->ldap != NULL, FALSE);
     g_assert (lop->ldap_op == -1);
     
     filter = (gchar*)g_object_get_data (G_OBJECT (lop), "filter");
     g_return_val_if_fail (filter != NULL, FALSE);
+
+    t = (gchar*)g_object_get_data (G_OBJECT (lop), "details");
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0, 0);
     
     sinfo = get_ldap_server_info (lop->lsrc, TRUE);
     lop->ldap_op = ldap_search (lop->ldap, sinfo->base_dn, LDAP_SCOPE_SUBTREE,
@@ -645,14 +719,14 @@ start_search_operation (SeahorseLDAPSource *lsrc, const gchar *pattern)
     filter = g_strdup_printf ("(pgpuserid=*%s*)", t);
     g_free (t);
     
-    lop = seahorse_ldap_operation_start (lsrc, start_search);
+    lop = seahorse_ldap_operation_start (lsrc, start_search, 0);
     g_return_val_if_fail (lop != NULL, NULL);
-
-    t = g_strdup_printf (_("Searching for keys containing '%s'..."), pattern);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, -1);
-    g_free (t);
     
     g_object_set_data_full (G_OBJECT (lop), "filter", filter, g_free);
+    
+    t = g_strdup_printf (_("Searching for keys containing '%s'..."), pattern);
+    g_object_set_data_full (G_OBJECT (lop), "details", t, g_free);
+    
     return lop;
 }
 
@@ -661,7 +735,7 @@ static SeahorseLDAPOperation *
 start_search_operation_fpr (SeahorseLDAPSource *lsrc, const gchar *fpr)
 {
     SeahorseLDAPOperation *lop;
-    gchar *filter;
+    gchar *filter, *t;
     guint l;
     
     g_return_val_if_fail (fpr && fpr[0], NULL);
@@ -672,10 +746,14 @@ start_search_operation_fpr (SeahorseLDAPSource *lsrc, const gchar *fpr)
     
     filter = g_strdup_printf ("(pgpcertid=%.16s)", fpr);
     
-    lop = seahorse_ldap_operation_start (lsrc, start_search);
+    lop = seahorse_ldap_operation_start (lsrc, start_search, 1);
     g_return_val_if_fail (lop != NULL, NULL);
     
     g_object_set_data_full (G_OBJECT (lop), "filter", filter, g_free);
+    
+    t = g_strdup_printf (_("Searching for key id '%s'..."), fpr);
+    g_object_set_data_full (G_OBJECT (lop), "details", t, g_free);
+    
     return lop;
 }
 
@@ -692,7 +770,6 @@ get_callback (SeahorseOperation *op, LDAPMessage *result)
     SeahorseLDAPOperation *lop = SEAHORSE_LDAP_OPERATION (op);  
     LDAPServerInfo *sinfo;
     gpgme_data_t data;
-    gpgme_error_t err;
     char *message;
     gchar *key;
     int code;
@@ -705,7 +782,11 @@ get_callback (SeahorseOperation *op, LDAPMessage *result)
      
     /* An LDAP Entry */
     if (r == LDAP_RES_SEARCH_ENTRY) {
-           
+        
+#ifdef _DEBUG
+        g_printerr ("[ldap] Retrieved Key Data:\n");
+        dump_ldap_entry (lop->ldap, result);
+#endif                 
         key = get_string_attribute (lop->ldap, result, sinfo->key_attr);
         
         if (key == NULL) {
@@ -714,15 +795,11 @@ get_callback (SeahorseOperation *op, LDAPMessage *result)
         }
         
         data = (gpgme_data_t)g_object_get_data (G_OBJECT (lop), "result");
-        if (data == NULL) {
-            err = gpgme_data_new (&data);
-            g_return_val_if_fail (data != NULL, FALSE);
-            g_object_set_data_full (G_OBJECT (lop), "result", data, 
-                                    (GDestroyNotify)gpgme_data_release);
-        }
-
+        g_return_val_if_fail (data != NULL, FALSE);
+        
         r = gpgme_data_write (data, key, strlen (key));
         g_return_val_if_fail (r != -1, FALSE);
+
         g_free (key);
         return TRUE;
 
@@ -754,7 +831,7 @@ get_key_from_ldap (SeahorseOperation *op, LDAPMessage *result)
 {
     SeahorseLDAPOperation *lop = SEAHORSE_LDAP_OPERATION (op);  
     LDAPServerInfo *sinfo;
-    GSList *fingerprints;
+    GSList *fingerprints, *fprfull;
     gchar *filter;
     char *attrs[2];
     const gchar *fpr;
@@ -764,6 +841,12 @@ get_key_from_ldap (SeahorseOperation *op, LDAPMessage *result)
     g_assert (lop->ldap_op == -1);
     
     fingerprints = (GSList*)g_object_get_data (G_OBJECT (lop), "fingerprints");
+    fprfull = (GSList*)g_object_get_data (G_OBJECT (lop), "fingerprints-full");
+
+    l = g_slist_length (fprfull);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), 
+                                      _("Retrieving remote keys..."), 
+                                      l - g_slist_length (fingerprints), l);
     
     if (fingerprints) {
      
@@ -809,24 +892,29 @@ start_get_operation_multiple (SeahorseLDAPSource *lsrc, GSList *fingerprints,
                               gpgme_data_t data)
 {
     SeahorseLDAPOperation *lop;
-
+    gpgme_error_t gerr;
+    
     g_return_val_if_fail (g_slist_length (fingerprints) > 0, NULL);
     
-    lop = seahorse_ldap_operation_start (lsrc, get_key_from_ldap);
+    lop = seahorse_ldap_operation_start (lsrc, get_key_from_ldap, 
+                                         g_slist_length(fingerprints));
     g_return_val_if_fail (lop != NULL, NULL);
-    
-    
+        
     if (data) {
         /* Note that we don't auto-free this */
         g_object_set_data (G_OBJECT (lop), "result", data);
+    } else {
+        /* But when we auto create a data object then we free it */
+        gerr = gpgme_data_new (&data);
+        g_return_val_if_fail (data != NULL, NULL);
+        g_object_set_data_full (G_OBJECT (lop), "result", data, 
+                                (GDestroyNotify)gpgme_data_release);
     }
     
-    /* TODO: We can update a percentage when there's more than one key */
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), _("Retrieving remote keys..."), -1);
-        
     g_object_set_data (G_OBJECT (lop), "fingerprints", fingerprints);
     g_object_set_data_full (G_OBJECT (lop), "fingerprints-full", fingerprints, 
                             (GDestroyNotify)seahorse_util_string_slist_free);
+
     return lop;
 }
 
@@ -861,6 +949,10 @@ send_callback (SeahorseOperation *op, LDAPMessage *result)
 
     r = ldap_parse_result (lop->ldap, result, &code, NULL, &message, NULL, NULL, 0);
     g_return_val_if_fail (r == LDAP_SUCCESS, FALSE);
+    
+    /* TODO: Somehow communicate this to the user */
+    if (code == LDAP_ALREADY_EXISTS)
+        code = LDAP_SUCCESS;
         
     if (code != LDAP_SUCCESS) 
         seahorse_operation_mark_done (SEAHORSE_OPERATION (lop), FALSE, 
@@ -881,17 +973,24 @@ send_key_to_ldap (SeahorseOperation *op, LDAPMessage *result)
 {
     SeahorseLDAPOperation *lop = SEAHORSE_LDAP_OPERATION (op);  
     LDAPServerInfo *sinfo;
-    GSList *keys;
+    GSList *keys, *keysfull;
     gchar *key;
     gchar *base;
     LDAPMod mod;
     LDAPMod *attrs[2];
     char *values[2];
+    guint l;
 
     g_return_val_if_fail (lop->ldap != NULL, FALSE);
     g_assert (lop->ldap_op == -1);
     
     keys = (GSList*)g_object_get_data (G_OBJECT (lop), "key-data");
+    keysfull = (GSList*)g_object_get_data (G_OBJECT (lop), "key-data-full");
+    
+    l = g_slist_length (keysfull);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), 
+                                      _("Sending keys to key server..."), 
+                                      l - g_slist_length (keys), l);
     
     if (keys) {
      
@@ -943,15 +1042,14 @@ start_send_operation_multiple (SeahorseLDAPSource *lsrc, GSList *keys)
 
     g_return_val_if_fail (g_slist_length (keys) > 0, NULL);
     
-    lop = seahorse_ldap_operation_start (lsrc, send_key_to_ldap);
+    lop = seahorse_ldap_operation_start (lsrc, send_key_to_ldap, 
+                                         g_slist_length (keys));
     g_return_val_if_fail (lop != NULL, NULL);
 
-    /* TODO: We can update a percentage when there's more than one key */    
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), _("Sending keys to key server..."), -1);
-    
     g_object_set_data (G_OBJECT (lop), "key-data", keys);
     g_object_set_data_full (G_OBJECT (lop), "key-data-full", keys, 
                             (GDestroyNotify)seahorse_util_string_slist_free);
+
     return lop;
 }
 
@@ -978,6 +1076,7 @@ static SeahorseOperation*  seahorse_ldap_source_import    (SeahorseKeySource *sk
                                                            gpgme_data_t data);
 static SeahorseOperation*  seahorse_ldap_source_export    (SeahorseKeySource *sksrc, 
                                                            GList *keys,     
+                                                           gboolean complete,
                                                            gpgme_data_t data);
                                                            
 static SeahorseKeySourceClass *parent_class = NULL;
@@ -1033,7 +1132,7 @@ seahorse_ldap_source_refresh (SeahorseKeySource *src, const gchar *key)
     
     /* No way to find new keys */
     if (g_str_equal (key, SEAHORSE_KEY_SOURCE_NEW))
-        return seahorse_operation_new_complete ();
+        return seahorse_operation_new_complete (NULL);
         
     /* All keys, parent_class will have cleared */
     else if(g_str_equal (key, SEAHORSE_KEY_SOURCE_ALL)) {
@@ -1142,7 +1241,7 @@ seahorse_ldap_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
 
 static SeahorseOperation* 
 seahorse_ldap_source_export (SeahorseKeySource *sksrc, GList *keys, 
-                             gpgme_data_t data)
+                             gboolean complete, gpgme_data_t data)
 {
     SeahorseLDAPOperation *lop;
     SeahorseLDAPSource *lsrc;
@@ -1184,7 +1283,6 @@ seahorse_ldap_source_new (SeahorseKeySource *locsrc, const gchar *server,
     g_return_val_if_fail (SEAHORSE_IS_KEY_SOURCE (locsrc) && 
                           !SEAHORSE_IS_SERVER_SOURCE (locsrc), NULL);
     g_return_val_if_fail (server && server[0], NULL);
-    g_return_val_if_fail (pattern && pattern[0], NULL);
     
     lsrc = g_object_new (SEAHORSE_TYPE_LDAP_SOURCE, "local-source", locsrc,
                          "key-server", server, "pattern", pattern, NULL);
