@@ -127,6 +127,7 @@ static SeahorseOperation*  seahorse_pgp_source_import           (SeahorseKeySour
                                                                  gpgme_data_t data);
 static SeahorseOperation*  seahorse_pgp_source_export           (SeahorseKeySource *sksrc, 
                                                                  GList *keys,
+                                                                 gboolean complete, 
                                                                  gpgme_data_t data);
 
 /* Other forward decls */
@@ -629,7 +630,7 @@ keyload_handler (SeahorseLoadOperation *lop)
     
     /* TODO: We can use the GPGME progress to make this more accurate */
     t = g_strdup_printf (ngettext("Loaded %d key", "Loaded %d keys", lop->loaded), lop->loaded);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, -1);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0, 0);
     g_free (t);
     
     return TRUE; 
@@ -693,7 +694,7 @@ seahorse_load_operation_start (SeahorsePGPSource *psrc, const gchar *pattern,
     }
     
     seahorse_operation_mark_start (SEAHORSE_OPERATION (lop));
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), _("Loading Keys..."), -1);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), _("Loading Keys..."), 0, 0);
     
     /* Run one iteration of the handler */
     keyload_handler (lop);
@@ -888,7 +889,7 @@ seahorse_pgp_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
 
 static SeahorseOperation* 
 seahorse_pgp_source_export (SeahorseKeySource *sksrc, GList *keys, 
-                            gpgme_data_t data)
+                            gboolean complete, gpgme_data_t data)
 {
     SeahorseOperation *operation;
     SeahorsePGPSource *psrc;
@@ -937,6 +938,14 @@ seahorse_pgp_source_export (SeahorseKeySource *sksrc, GList *keys,
 
         if (!GPG_IS_OK (gerr))
             break;
+        
+        if (complete && SEAHORSE_IS_KEY_PAIR (skey)) {
+            gerr = gpgmex_op_export_secret (new_ctx, seahorse_key_pair_get_id (SEAHORSE_KEY_PAIR (skey)), 
+                                            data);
+            
+            if (!GPG_IS_OK (gerr))
+                break;
+        }
     }
 
     if (!GPG_IS_OK (gerr)) 
