@@ -2,6 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2003 Jacob Perkins
+ * Copyright (C) 2004-2005 Nate Nielsen
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +26,7 @@
 #include <sys/wait.h>
 
 #include <gnome.h>
+#include <eel/eel.h>
 #include <libgnomevfs/gnome-vfs.h>
 
 #include "seahorse-gpgmex.h"
@@ -61,6 +63,8 @@ static const SeahorsePGPHeader seahorse_pgp_headers[] = {
     }
 };
 
+static const gchar *bad_filename_chars = "/\\<>|";
+
 void
 seahorse_util_show_error (GtkWindow *parent, const gchar *message)
 {
@@ -393,6 +397,34 @@ seahorse_util_detect_text (const gchar *text, gint len, const gchar **start,
     return SEAHORSE_TEXT_TYPE_NONE;
 }
 
+gchar*      
+seahorse_util_filename_for_keys (GList *keys)
+{
+    SeahorseKey *skey;
+    gchar *t, *r;
+    
+    g_return_val_if_fail (g_list_length (keys) > 0, NULL);
+
+    if (g_list_length (keys) == 1) {
+        g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), NULL);
+        skey = SEAHORSE_KEY (keys->data);
+        t = seahorse_key_get_userid_name (skey, 0);
+        if (t == NULL) {
+            t = g_strdup (seahorse_key_get_id (skey->key));
+            if (strlen (t) > 8)
+                t[8] = 0;
+        }
+    } else {
+        t = g_strdup (_("Multiple Keys"));
+    }
+    
+    g_strstrip (t);
+    g_strdelimit (t, bad_filename_chars, '_');
+    r = g_strconcat (t, SEAHORSE_EXT_ASC, NULL);
+    g_free (t);
+    return r;
+}
+
 /** 
  * seahorse_util_uri_get_last:
  * @uri: The uri to parse
@@ -740,7 +772,7 @@ void
 seahorse_util_chooser_show_key_files (GtkWidget *dialog)
 {
     GtkFileFilter* filter = gtk_file_filter_new ();
-    gtk_file_filter_set_name (filter, _("All PGP key files"));
+    gtk_file_filter_set_name (filter, _("All key files"));
     gtk_file_filter_add_mime_type (filter, "application/pgp-keys");    
     gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);    
     gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
@@ -790,7 +822,19 @@ seahorse_util_chooser_show_archive_files (GtkWidget *dialog)
     gtk_file_filter_add_pattern (filter, "*");    
     gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);   
 }
-        
+
+void
+seahorse_util_chooser_set_filename (GtkWidget *dialog, GList *keys)
+{
+    gchar *t = NULL;
+    
+    if (g_list_length (keys) > 0) {
+        t = seahorse_util_filename_for_keys (keys);
+        gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), t);
+        g_free (t);
+    }
+}
+    
 gchar*      
 seahorse_util_chooser_save_prompt (GtkWidget *dialog)
 {
@@ -855,15 +899,12 @@ seahorse_util_chooser_open_prompt (GtkWidget *dialog)
 gboolean
 seahorse_util_check_suffix (const gchar *path, SeahorseSuffix suffix)
 {
-	gchar *ext;
-	
 	if (suffix == SEAHORSE_SIG_SUFFIX)
-		ext = SEAHORSE_EXT_SIG;
+        return g_str_has_suffix (path, SEAHORSE_EXT_SIG);
 	else
-		ext = SEAHORSE_EXT_GPG;
-	
-	return (g_pattern_match_simple (g_strdup_printf ("*%s", SEAHORSE_EXT_ASC), path) ||
-		g_pattern_match_simple (g_strdup_printf ("*%s", ext), path));
+        return g_str_has_suffix (path, SEAHORSE_EXT_PGP) ||
+               g_str_has_suffix (path, SEAHORSE_EXT_GPG) ||
+               g_str_has_suffix (path, SEAHORSE_EXT_ASC);
 }
 
 /**
@@ -875,7 +916,7 @@ seahorse_util_check_suffix (const gchar *path, SeahorseSuffix suffix)
  *
  * Constructs a new path for a file based on @path plus a suffix determined by
  * @suffix and the ASCII Armor setting of @ctx. If ASCII Armor is enabled, the
- * suffix will be '.asc'. Otherwise the suffix will be '.gpg' if @suffix is
+ * suffix will be '.asc'. Otherwise the suffix will be '.pgp' if @suffix is
  * %SEAHORSE_CRYPT_SUFFIX or '.sig' if @suffix is %SEAHORSE_SIG_SUFFIX.
  *
  * Returns: A new path with the suffix appended to @path. NULL if prompt cancelled
@@ -889,16 +930,16 @@ seahorse_util_add_suffix (gpgme_ctx_t ctx, const gchar *path,
     gchar *uri;
     gchar *t;
 	
-	if (gpgme_get_armor (ctx) || suffix == SEAHORSE_ASC_SUFFIX)
+	if (suffix == SEAHORSE_CRYPT_SUFFIX)
+        ext = eel_gconf_get_boolean (ARMOR_KEY) ? SEAHORSE_EXT_ASC : SEAHORSE_EXT_PGP;
+	else if (suffix == SEAHORSE_ASC_SUFFIX)
 		ext = SEAHORSE_EXT_ASC;
-	else if (suffix == SEAHORSE_CRYPT_SUFFIX)
-		ext = SEAHORSE_EXT_GPG;
 	else
 		ext = SEAHORSE_EXT_SIG;
 	
 	uri = g_strdup_printf ("%s%s", path, ext);
     
-    if (prompt && seahorse_util_uri_exists (uri)) {
+    if (prompt && uri && seahorse_util_uri_exists (uri)) {
             
         t = g_strdup_printf (prompt, seahorse_util_uri_get_last (uri));
         dialog = seahorse_util_chooser_save_new (t, NULL);
@@ -934,7 +975,7 @@ seahorse_util_remove_suffix (const gchar *path, const gchar *prompt)
 
 	uri =  g_strndup (path, strlen (path) - 4);
    
-    if (prompt && seahorse_util_uri_exists (uri)) {
+    if (prompt && uri && seahorse_util_uri_exists (uri)) {
             
         t = g_strdup_printf (prompt, seahorse_util_uri_get_last (uri));
         dialog = seahorse_util_chooser_save_new (t, NULL);
@@ -1053,6 +1094,15 @@ seahorse_util_keylist_splice (GList *keys)
     return NULL;
 }
 
+gboolean    
+seahorse_util_string_equals (const gchar *s1, const gchar *s2)
+{
+    if (!s1 && !s2)
+        return TRUE;
+    if (!s1 || !s2)
+        return FALSE;
+    return g_str_equal (s1, s2);
+}
 
 /* Free a GSList along with string values */
 GSList*
@@ -1074,3 +1124,17 @@ seahorse_util_string_slist_copy (GSList *list)
         l = g_slist_append (l, g_strdup(list->data));
     return l;
 }
+
+/* Compare two string GSLists */
+gboolean    
+seahorse_util_string_slist_equal (GSList *l1, GSList *l2)
+{
+    while (l1 && l2) {
+        if (!g_str_equal ((const gchar*)l1->data, (const gchar*)l2->data))
+            break;
+        l1 = g_slist_next (l1);
+        l2 = g_slist_next (l2);
+    }
+    
+    return !l1 && !l2;   
+}
