@@ -1,7 +1,7 @@
 /*
  * Seahorse
  *
- * Copyright (C) 2004 Nate Nielsen
+ * Copyright (C) 2004-2005 Nate Nielsen
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #include <gedit/gedit-menus.h>
 #include <gedit/gedit-utils.h>
 
+#include "seahorse-gpgmex.h"
 #include "seahorse-util.h"
 #include "seahorse-context.h"
 #include "seahorse-libdialogs.h"
@@ -44,17 +45,17 @@
 #define MENU_ENC_ITEM_LABEL      N_("_Encrypt...")
 #define MENU_ENC_ITEM_PATH       "/menu/Edit/EditOps_6/"
 #define MENU_ENC_ITEM_NAME       "Encrypt"
-#define MENU_ENC_ITEM_TIP        N_("PGP Encrypt the selected text")
+#define MENU_ENC_ITEM_TIP        N_("Encrypt the selected text")
 
 #define MENU_DEC_ITEM_LABEL      N_("Decr_ypt/Verify")
 #define MENU_DEC_ITEM_PATH       "/menu/Edit/EditOps_6/"
 #define MENU_DEC_ITEM_NAME       "Decrypt"
-#define MENU_DEC_ITEM_TIP        N_("Decrypt and/or Verify PGP text")
+#define MENU_DEC_ITEM_TIP        N_("Decrypt and/or Verify text")
 
 #define MENU_SIGN_ITEM_LABEL      N_("Sig_n...")
 #define MENU_SIGN_ITEM_PATH       "/menu/Edit/EditOps_6/"
 #define MENU_SIGN_ITEM_NAME       "Sign"
-#define MENU_SIGN_ITEM_TIP        N_("PGP Sign the selected text")
+#define MENU_SIGN_ITEM_TIP        N_("Sign the selected text")
 
 static SeahorseContext *sctx = NULL;
 
@@ -136,19 +137,16 @@ replace_selected_text (GeditDocument *doc, const gchar *replace)
 {
     GtkTextIter iter;
     GtkTextIter sel_bound;
-    gchar *converted_replace;
 
     g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
     g_return_if_fail (replace != NULL);
 
     if (!gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (doc),           
                                                   &iter, &sel_bound)) {
-        gedit_debug (DEBUG_DOCUMENT, "There is no selected text");
+        gedit_debug (DEBUG_PLUGINS, "There is no selected text");
         return;
     }
 
-    converted_replace = gedit_utils_convert_search_text (replace);
-
     gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER (doc));
 
     gtk_text_buffer_delete_selection (GTK_TEXT_BUFFER (doc), FALSE, TRUE);
@@ -156,12 +154,13 @@ replace_selected_text (GeditDocument *doc, const gchar *replace)
     gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),            
                    &iter, gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (doc)));
 
-    if (*converted_replace != '\0')
+    g_printerr ("%s\n", replace);
+
+    if (*replace != '\0')
         gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc), &iter,
-                 converted_replace, strlen (converted_replace));
+                 replace, strlen (replace));
 
     gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (doc));
-    g_free (converted_replace);
 }
 
 /* -----------------------------------------------------------------------------
@@ -173,7 +172,8 @@ static void
 encrypt_cb (BonoboUIComponent * uic, gpointer user_data,
             const gchar * verbname)
 {
-    GeditView *view = gedit_get_active_view ();
+    GeditView *view = GEDIT_VIEW (gedit_get_active_view ());
+    SeahorseKeyPair *signer = NULL;
     GeditDocument *doc;
     gpgme_error_t err = GPG_OK;
     gchar *enctext = NULL;
@@ -185,35 +185,43 @@ encrypt_cb (BonoboUIComponent * uic, gpointer user_data,
     gedit_debug (DEBUG_PLUGINS, "");
     g_assert (SEAHORSE_IS_CONTEXT (sctx));
 
-    g_return_if_fail (view);
+    g_return_if_fail (view != NULL);
+    doc = gedit_view_get_document (view);
 
+    /* 
+     * We get the selection bounds before getting recipients.
+     * This is because in some cases the selection is removed
+     * once we start messing with recipients.
+     */
+    if (!get_document_selection (doc, &start, &end)) {
+        start = 0;
+        end = -1;
+    }
+    
     /* Get the recipient list */
     gedit_debug (DEBUG_PLUGINS, "getting recipients");
-    keys = seahorse_recipients_get (sctx);
+    keys = seahorse_recipients_get (sctx, &signer);
 
     /* User may have cancelled */
     if (g_list_length(keys) == 0)
         return;
 
-    /* Get the text */
-    doc = gedit_view_get_document (view);
-
-    if (get_document_selection (doc, &start, &end)) {
-        buffer = get_document_chars (doc, start, end);
-    } else {
-        buffer = get_document_chars (doc, 0, -1);
-    }
+    /* Get the document text */
+    buffer = get_document_chars (doc, start, end);    
 
     /* Encrypt it */
     gedit_debug (DEBUG_PLUGINS, "encrypting text");
-    enctext = seahorse_op_encrypt_text (keys, buffer, &err);
+    if (signer == NULL)
+        enctext = seahorse_op_encrypt_text (keys, buffer, &err);
+    else
+        enctext = seahorse_op_encrypt_sign_text (keys, signer, buffer, &err);
 
     g_list_free (keys);
     g_free (buffer);
 
     if (!GPG_IS_OK (err)) {
         g_assert (!enctext);
-        seahorse_util_handle_error (err, _("Couldn't encrypt text"));
+        seahorse_util_handle_gpgme (err, _("Couldn't encrypt text"));
         return;
     }
 
@@ -229,20 +237,18 @@ static guint
 import_keys (const gchar * text)
 {
     SeahorseKeySource *sksrc;
-    gpgme_error_t err;
+    GError *err = NULL;
     gint keys;
 
-    sksrc = seahorse_context_get_pri_source (sctx);
+    sksrc = seahorse_context_get_key_source (sctx);
     g_return_val_if_fail (sksrc != NULL, 0);
     
     keys = seahorse_op_import_text (sksrc, text, &err);
 
-    if (!GPG_IS_OK (err)) {
+    if (keys < 0) {
         seahorse_util_handle_error (err, _("Couldn't import keys"));
         return 0;
-    }
-
-    if (keys == 0) {    
+    } else if (keys == 0) {    
         gedit_utils_flash (_("Keys found but not imported"));
         return 0;
     } else {
@@ -258,13 +264,13 @@ decrypt_text (const gchar * text, gpgme_verify_result_t *status)
     gpgme_error_t err;
     gchar *rawtext = NULL;
     
-    sksrc = seahorse_context_get_pri_source (sctx);
+    sksrc = seahorse_context_get_key_source (sctx);
     g_return_val_if_fail (sksrc != NULL, 0);
 
     rawtext = seahorse_op_decrypt_verify_text (sksrc, text, status, &err);
 
     if (!GPG_IS_OK (err)) {
-        seahorse_util_handle_error (err, _("Couldn't decrypt text"));
+        seahorse_util_handle_gpgme (err, _("Couldn't decrypt text"));
         return NULL;
     }
 
@@ -279,13 +285,13 @@ verify_text (const gchar * text, gpgme_verify_result_t *status)
     gpgme_error_t err;
     gchar *rawtext = NULL;
 
-    sksrc = seahorse_context_get_pri_source (sctx);
+    sksrc = seahorse_context_get_key_source (sctx);
     g_return_val_if_fail (sksrc != NULL, 0);    
 
     rawtext = seahorse_op_verify_text (sksrc, text, status, &err);
 
     if (!GPG_IS_OK (err)) {
-        seahorse_util_handle_error (err, _("Couldn't decrypt text"));
+        seahorse_util_handle_gpgme (err, _("Couldn't decrypt text"));
         return NULL;
     }
 
@@ -297,7 +303,7 @@ static void
 decrypt_cb (BonoboUIComponent * uic, gpointer user_data,
             const gchar * verbname)
 {
-    GeditView *view = gedit_get_active_view ();
+    GeditView *view = GEDIT_VIEW (gedit_get_active_view ());
     GeditDocument *doc;
     
     gchar *buffer;              /* The text selected */
@@ -348,7 +354,7 @@ decrypt_cb (BonoboUIComponent * uic, gpointer user_data,
         if (type == SEAHORSE_TEXT_TYPE_NONE) {
             if (blocks == 0) 
                 gedit_warning (GTK_WINDOW (gedit_get_active_window ()),
-                           _("No PGP encrypted or signed text is selected."));
+                           _("No encrypted or signed text is selected."));
             break;
         }
         
@@ -445,7 +451,7 @@ static void
 sign_cb (BonoboUIComponent * uic, gpointer user_data,
          const gchar * verbname)
 {
-    GeditView *view = gedit_get_active_view ();
+    GeditView *view = GEDIT_VIEW (gedit_get_active_view ());
     GeditDocument *doc;
     gpgme_error_t err = GPG_OK;
     SeahorseKeyPair *signer;
@@ -458,8 +464,6 @@ sign_cb (BonoboUIComponent * uic, gpointer user_data,
     g_assert (SEAHORSE_IS_CONTEXT (sctx));
 
     g_return_if_fail (view);
-
-    /* Get the document text */
     doc = gedit_view_get_document (view);
 
     if (!get_document_selection (doc, &start, &end)) {
@@ -467,13 +471,12 @@ sign_cb (BonoboUIComponent * uic, gpointer user_data,
         end = -1;
     }
 
+    /* Get the document text */
     buffer = get_document_chars (doc, start, end);
 
-    signer = seahorse_context_get_default_key (sctx);
-    if (signer == NULL) {
-        seahorse_util_handle_error (GPG_E (GPG_ERR_NO_SECKEY), _("Couldn't sign text"));
+    signer = seahorse_signer_get (sctx);
+    if (signer == NULL)
         return;
-    }
 
     /* Perform the signing */
     gedit_debug (DEBUG_PLUGINS, "signing text");
@@ -482,7 +485,7 @@ sign_cb (BonoboUIComponent * uic, gpointer user_data,
 
     if (!GPG_IS_OK (err)) {
         g_assert (!enctext);
-        seahorse_util_handle_error (err, _("Couldn't sign text"));
+        seahorse_util_handle_gpgme (err, _("Couldn't sign text"));
         return;
     }
 
