@@ -21,8 +21,10 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <locale.h>
 
+#include "seahorse-gpgmex.h"
 #include "seahorse-context.h"
 #include "seahorse-windows.h"
 #include "seahorse-op.h"
@@ -124,14 +126,14 @@ do_import (SeahorseContext *sctx, const gchar **paths)
 {
     SeahorseKeySource *sksrc;
     GtkWidget *dlg;
-    gpgme_error_t err;
+    GError *err = NULL;
     gint keys = 0;
     gchar **uris;
     gchar **u;
     gchar *t;
     guint ret = 0;
     
-    sksrc = seahorse_context_get_pri_source (sctx);
+    sksrc = seahorse_context_get_key_source (sctx);
     g_return_val_if_fail (sksrc != NULL, 1);
     
     /* Get all sub-folders etc... */
@@ -141,7 +143,7 @@ do_import (SeahorseContext *sctx, const gchar **paths)
     for (u = uris; *u; u++) {
         keys += seahorse_op_import_file (sksrc, *u, &err);
        
-        if (!GPG_IS_OK (err)) {
+        if (err != NULL) {
             seahorse_util_handle_error (err, _("Couldn't import keys from \"%s\""),
                     seahorse_util_uri_get_last (*u));
             ret = 1;
@@ -164,8 +166,10 @@ do_import (SeahorseContext *sctx, const gchar **paths)
 
 /* Encrypt or sign the given set of paths */
 static guint 
-do_encrypt_base (SeahorseContext *sctx, const gchar **paths, SeahorseKeyPair *signer)
+do_encrypt_base (SeahorseContext *sctx, const gchar **paths, gboolean sign)
 {
+    SeahorseKeyPair *signer = NULL;
+    SeahorseKeySource *sksrc;
     GList *keys = NULL;
     gchar *new_path;
     gpgme_error_t err;
@@ -173,25 +177,34 @@ do_encrypt_base (SeahorseContext *sctx, const gchar **paths, SeahorseKeyPair *si
     gchar **u;
     guint ret = 0;
 
+    sksrc = seahorse_context_get_key_source (sctx);
+    g_return_val_if_fail (sksrc != NULL, 1);
+    
     /* This can change the list of uris and optionally compress etc... */
     uris = seahorse_process_multiple (sctx, paths, NULL);
 
     if(!uris)
         ret = 1;
     else {
-        keys = seahorse_recipients_get (sctx);
+        keys = seahorse_recipients_get (sctx, sign ? &signer : NULL);
            
         if (g_list_length (keys) > 0) {
             for(u = uris; *u; u++) {
                
+                new_path = seahorse_util_add_suffix (sksrc->ctx, *u, SEAHORSE_CRYPT_SUFFIX, 
+                                                     _("Choose Encrypted File Name for '%s'"));
+                if (!new_path) 
+                    break;
+                    
                 if (signer)
-                    new_path = seahorse_op_encrypt_sign_file (keys, signer, *u, &err);
+                    seahorse_op_encrypt_sign_file (keys, signer, *u, new_path, &err);
                 else
-                    new_path = seahorse_op_encrypt_file (keys, *u, &err);
+                    seahorse_op_encrypt_file (keys, *u, new_path, &err);
+                
                 g_free (new_path);
     
                 if (!GPG_IS_OK (err)) {
-                    seahorse_util_handle_error (err, _("Couldn't encrypt \"%s\""), 
+                    seahorse_util_handle_gpgme (err, _("Couldn't encrypt \"%s\""), 
                         seahorse_util_uri_get_last (*u));
                     ret = 1;
                     break;
@@ -211,14 +224,14 @@ do_encrypt_base (SeahorseContext *sctx, const gchar **paths, SeahorseKeyPair *si
 static guint
 do_encrypt (SeahorseContext *sctx, const gchar **paths)
 {
-    return do_encrypt_base (sctx, paths, NULL);
+    return do_encrypt_base (sctx, paths, FALSE);
 }
 
 /* Encrypt and sign the given set of paths */
 static guint
 do_encrypt_sign (SeahorseContext *sctx, const gchar **paths)
 {
-    return do_encrypt_base (sctx, paths, seahorse_context_get_default_key (sctx));
+    return do_encrypt_base (sctx, paths, TRUE);
 }
 
 static guint
@@ -232,21 +245,28 @@ do_sign (SeahorseContext *sctx, const gchar **paths)
     gchar *new_path;
     guint ret = 0;
     
-    sksrc = seahorse_context_get_pri_source (sctx);
+    sksrc = seahorse_context_get_key_source (sctx);
     g_return_val_if_fail (sksrc != NULL, 1);
-    
-    signer = seahorse_context_get_default_key (sctx);
-    g_return_val_if_fail (signer != NULL, 1);
+
+    signer = seahorse_signer_get (sctx);    
+    if (signer == NULL)
+        return 1;
     
     uris = seahorse_util_uris_expand (paths);
     g_assert (uris != NULL);
      
     for (u = uris; *u; u++) {
-        new_path = seahorse_op_sign_file (signer, *u, &err);
+      
+        new_path = seahorse_util_add_suffix (sksrc->ctx, *u, SEAHORSE_SIG_SUFFIX, 
+                                             _("Choose Signature File Name for '%s'"));
+        if (!new_path)
+            break;
+        
+        seahorse_op_sign_file (signer, *u, new_path, &err);
         g_free(new_path);
         
         if (!GPG_IS_OK (err)) {
-            seahorse_util_handle_error (err, _("Couldn't sign \"%s\""),
+            seahorse_util_handle_gpgme (err, _("Couldn't sign \"%s\""),
                 seahorse_util_uri_get_last (*u));
             ret = 1;
             break;
@@ -270,22 +290,27 @@ do_decrypt (SeahorseContext *sctx, const gchar **paths)
     gchar *new_path;
     guint ret = 0;
 	
-    sksrc = seahorse_context_get_pri_source (sctx);
+    sksrc = seahorse_context_get_key_source (sctx);
     g_return_val_if_fail (sksrc != NULL, 1);
         
     uris = seahorse_util_uris_expand (paths);
     g_assert (uris != NULL);
      
     for (u = uris; *u; u++) {
-        new_path = seahorse_op_decrypt_verify_file (sksrc, *u, &status, &err);
+      
+        new_path = seahorse_util_remove_suffix (*u, _("Choose Decrypted File Name for '%s'"));
+        if(!new_path)
+            break;
+            
+        seahorse_op_decrypt_verify_file (sksrc, *u, new_path, &status, &err);
                 
         if (!GPG_IS_OK (err)) {
-            seahorse_util_handle_error (err, _("Couldn't decrypt \"%s\""),
+            seahorse_util_handle_gpgme (err, _("Couldn't decrypt \"%s\""),
                     seahorse_util_uri_get_last (*u));
             ret = 1;
             break;
         }
-        
+    
         if(status && status->signatures) {
             if(!signatures)
                 signatures = seahorse_signatures_new (sctx);
@@ -319,12 +344,12 @@ do_verify (SeahorseContext *sctx, const gchar **paths)
     uris = seahorse_util_uris_expand (paths);
     g_assert (uris != NULL);
 	
-    sksrc = seahorse_context_get_pri_source (sctx);
+    sksrc = seahorse_context_get_key_source (sctx);
     g_return_val_if_fail (sksrc != NULL, 1);
          
     for (u = uris; *u; u++) {
 
-        original = seahorse_util_remove_suffix (*u);
+        original = seahorse_util_remove_suffix (*u, NULL);
 
         /* The original file doesn't exist, prompt for it */
         if (!seahorse_util_uri_exists (original)) {
@@ -360,7 +385,7 @@ do_verify (SeahorseContext *sctx, const gchar **paths)
             g_free (original);
 
             if (!GPG_IS_OK (err)) {
-                seahorse_util_handle_error (err, _("Couldn't verify \"%s\""),
+                seahorse_util_handle_gpgme (err, _("Couldn't verify \"%s\""),
                         seahorse_util_uri_get_last (*u));
                 ret = 1;
                 break;
@@ -406,7 +431,7 @@ main (int argc, char **argv)
 
     program = gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
                     GNOME_PARAM_POPT_TABLE, options,
-                    GNOME_PARAM_HUMAN_READABLE_NAME, _("GPG Keys Manager"),
+                    GNOME_PARAM_HUMAN_READABLE_NAME, _("Encryption Key Manager"),
                     GNOME_PARAM_APP_DATADIR, DATA_DIR, NULL);
 
     sctx = seahorse_context_new ();
@@ -445,13 +470,15 @@ main (int argc, char **argv)
                 
             g_strfreev (uris);
         }
-        
-        return ret;
-    }
- 
-    win = seahorse_key_manager_show (sctx);
-    g_signal_connect_after (G_OBJECT (win), "destroy", gtk_main_quit, NULL);
-	gtk_main ();
-	
-	return 0;
+
+    } else { 
+        win = seahorse_key_manager_show (sctx);
+        g_signal_connect_after (G_OBJECT (win), "destroy", gtk_main_quit, NULL);
+	    gtk_main ();
+	}
+    
+    if (gnome_vfs_initialized ())
+        gnome_vfs_shutdown ();
+    
+	return ret;
 }
